/*******************************************************************************
 *
 * Copyright 2012-2013 NetBurner, Inc.  ALL RIGHTS RESERVED
 *
 * Permission is hereby granted to purchasers of NetBurner hardware to use or
 * modify this computer program for any use as long as the resultant program is
 * only executed on NetBurner-provided hardware.
 *
 * No other rights to use this program or its derivatives in part or in whole
 * are granted.
 *
 * It may be possible to license this or other NetBurner software for use on
 * non-NetBurner hardware.  Please contact licensing@netburner.com for more
 * information.
 *
 * NetBurner makes no representation or warranties with respect to the
 * performance of this computer program, and specifically disclaims any
 * responsibility for any damages, special or consequential, connected with use
 * of this program.
 *
 * ----------------------------------------------------------------------------
 *
 * NetBurner, Inc.
 * 5405 Morehouse Drive
 * San Diego, CA  92121
 *
 * Information available at:  http://www.netburner.com
 * Support available at:      http://support.netburner.com
 *
 ******************************************************************************/
/*------------------------------------------------------------------------------
 * MCF4415 IRQ Demo
 * This example program uses a GPIO pin to trigger an external interrupt
 * pin. A jumper must be placed from the GPIO pin to the desired interrupt
 * input. To run the example use MTTTY or some other serial terminal to
 * interact with the menu to trigger the interrupt and view the interrupt
 * counter.
 *
 * The MCF54415 processor is used on both the MOD54415 and NANO54415 modules.
 * The table below list the available interrupts and GPIO jumper configuration.
 *
 * MOD54415:
 *    GPIO signal pin: J2.33
 *    IRQ 1: pin 45
 *    IRQ 2: pin 43
 *    IRQ 3: pin 26
 *    IRQ 6: pin 47
 *
 * NANO54415:
 *    GPIO signal:  Nano pin 12 = Dev board pin P2.11
 *    IRQ 2: Nano pin 50 = Dev board pin P1.50
 *    IRQ 3: Nano pin 49 = Dev board pin P1.49
 *
 *
 *----------------------------------------------------------------------------*/

#include "predef.h"
#include <stdio.h>
#include <ctype.h>
#include <string.h>
#include <startnet.h>
#include <autoupdate.h>
#include <dhcpclient.h>
#include <networkdebug.h>
#include <pins.h>
#include <pin_irq.h>
#include <smarttrap.h>

#ifdef MOD5441X
#include <..\MOD5441X\include\sim5441x.h>
#elif NANO54415
#include <..\NANO54415\include\sim5441x.h>
#else
#error "Platform not suppored in example"
#endif


const char *AppName = "MCF54415 IRQ Demo";

extern "C"
{
   void UserMain( void *pd );
}

unsigned int IRQ_counter;


/*-----------------------------------------------------------------------------
 * Display debug menu.
 *-----------------------------------------------------------------------------*/
void DisplayMenu( void )
{
   iprintf( "\r\n----- Main Menu -----\r\n" );
   iprintf( "I - Trigger interrupt by writing GPIO low\r\n" );
   iprintf( "? - Display Menu\r\n" );
   iprintf( "<enter> - Display interrupt count\r\n" );
   iprintf("Number of interrupts: %d\r\n", IRQ_counter );
}


/*-----------------------------------------------------------------------------
 * Handle commands with a trivially simple command dispatcher.
 *-----------------------------------------------------------------------------*/
void ProcessCommand( char c )
{
   switch ( toupper( c ) )
   {
      case 'T':
      case 't':
         #ifdef MOD5441X
            J2[33] = 0;
         #elif NANO54415
            Pins[12] = 0;
         #endif
         break;
      case 'H':
      case 'h':
         #ifdef MOD5441X
            J2[33] = 1;
         #elif NANO54415
            Pins[12] = 1;
         #endif
         break;
     default: // '?'
         DisplayMenu();
   }
}




/*-----------------------------------------------------------------------------
 * Ineterrupt Service Routine
 * Called when the interrupt occurs. The flag in the Edge Port Flag Register
 * is automatically cleared by the code in the SetPinIrq() implementation.
 *
 *-----------------------------------------------------------------------------*/
void EdgeportISR()
{
#ifdef MOD5441X
   J2[33] = 1;    // Set GPIO pin high
#elif NANO54415
   Pins[12] = 1;  // Set GPIO pin high
#endif


   IRQ_counter++;
}


/*-----------------------------------------------------------------------------
 * UserMain
 *-----------------------------------------------------------------------------*/
void UserMain( void *pd )
{

   InitializeStack();   // Initialize TCP/IP stack

   if ( EthernetIP == 0 )
   {
      iprintf( "Trying DHCP...\r\n" );
      GetDHCPAddress();
      iprintf( "DHCP assigned the IP address of:  " );
      ShowIP( EthernetIP );
      iprintf( "\r\n" );
   }
   else
   {
      iprintf( "Static IP address:  " );
      ShowIP( EthernetIP );
      iprintf( "\r\n" );
   }

   OSChangePrio( MAIN_PRIO );   // Set main task priority to the default (50)
   EnableAutoUpdate();          // Enable NB application AutoUpdate support
   EnableSmartTraps();

#ifdef MOD5441X
   // GPIO pin used to trigger IRQ
   J2[33].function( PINJ2_33_GPIO );
   J2[33] = 1;

   // Uncomment irq you want to use
   SetPinIrq( 26, -1, &EdgeportISR);  // IRQ 3
   //SetPinIrq( 43, -1, &EdgeportISR);  // IRQ 2
   //SetPinIrq( 45, -1, &EdgeportISR);  // IRQ 1
   //SetPinIrq( 47, -1, &EdgeportISR);  // IRQ 6
#elif NANO54415
   // GPIO pin used to trigger IRQ
   Pins[12].function( PIN_12_GPIO );
   Pins[12] = 1;

   // Uncomment irq you want to use
   SetPinIrq( 49, -1, &EdgeportISR);  // IRQ 3
   //SetPinIrq( 50, -1, &EdgeportISR);  // IRQ 2
#else
#error "Platform not suppored in example"
#endif

   IRQ_counter = 0;
   DisplayMenu();

   while ( 1 )
   {
      if ( charavail() )
      {
         char c = getchar();
         ProcessCommand( c );
      }
   }
}
